@@ -3,13 +3,6 @@ isNA <- function(x) {
   is.na(x)
 }
 
-isFALSE <- function(x) {
-  if (length(x) != 1L) return(FALSE)
-  x <- as.logical(x)
-  x <- unclass(x)
-  identical(FALSE, x)
-}
-
 attachedPackages <- function() {
   pkgs <- search()
   pkgs <- grep("^package:", pkgs, value=TRUE)
